//20250330
//本例程基于Ai8051U核心转接板(擎天柱) 进行编写测试。
//用于测试USB传输中的CRC5的计算,计算结果通过USB-CDC上送PC机串口调试器显示
//使用STC的CDC库，KC251编译,烧写程序时选择24MHz时钟;
//
//==================================================================================================
//CRC5-USB测试结果记录:	[USB中传输的时候是LSB在前的] 【参见: www.usbzh.com 中usb各种包格式例程中的 样本数据】
//7bit-addr						4bit-endp					CRC5计算源数据	
//实际值=2		传输值=0x20		实际值=0	传输值=0x0		0x200=传输值	CRC5_MSBfirst()		CRC5_LSBfirst()=crc5_usb_11bit_input()
//000 0010		010 0000		0000		0000			010 0000 0000		0x15				0x15
//
//实际值=23		传输值=0x74		实际值=0	传输值=0x0		0x740=传输值	CRC5_MSBfirst()		CRC5_LSBfirst()=crc5_usb_11bit_input()
//001 0111		111 0100		0000		0000			111 0100 0000		0x11				0x11			
//
//实际值=0		传输值=0x00		实际值=0	传输值=0x0		0x000=传输值	CRC5_MSBfirst()		CRC5_LSBfirst()=crc5_usb_11bit_input()
//000 0000		000 0000		0000		0000			000 0000 0000		0x08				0x02
//
//实际值=1		传输值=0x40		实际值=1	传输值=0x1		0x401=传输值	CRC5_MSBfirst()		CRC5_LSBfirst()=crc5_usb_11bit_input()
//000 0001		100 0000		0001		0001			100 0000 0001		0x12				0x09
//
//实际值=1		传输值=0x40		实际值=1	传输值=0x8		0x408=传输值	CRC5_MSBfirst()		CRC5_LSBfirst()=crc5_usb_11bit_input()
//000 0001		100 0000		0001		1000			100 0000 1000		0x1A				0x18
//
//实际值=15		传输值=0x78		实际值=2	传输值=0x4		0x784=传输值	CRC5_MSBfirst()		CRC5_LSBfirst()=crc5_usb_11bit_input()
//000 1111		111 1000		0010		0100			100 0000 1000		0x0C				0x08
//
//测试结果说明: 
//	1. 使用楼主的函数 CRC5_MSBfirst(),计算CRC5成功，返回的就是USB传输值, 符合样本的CRC5结果!  
//	2. CRC5_LSBfirst() 和 crc5_usb_11bit_input() 的计算结果一样, 不 是样本的CRC5结果!
//==================================================================================================

#define PRINTF_HID
#include "../comm/Ai8051U.h"  //包含此头文件后，不需要再包含"reg51.h"头文件
#include "../comm/usb.h"     //USB调试及复位所需头文件
#include "intrins.h"
#include "stdio.h"

#define MAIN_Fosc     24000000L  //定义主时钟
#define Baudrate      115200L
#define TM            (65536 -(MAIN_Fosc/Baudrate/4))

#define Timer0_Reload (65536UL -(MAIN_Fosc / 1000))       //Timer 0 中断频率, 1000次/秒

/*************	本地变量声明	**************/

//USB调试及复位所需定义
char *USER_DEVICEDESC = NULL;
char *USER_PRODUCTDESC = NULL;
char *USER_STCISPCMD = "@STCISP#";                      //设置自动复位到ISP区的用户接口命令

//P3.2口按键复位所需变量
bit Key_Flag;
u16 Key_cnt;


void Timer0_Init(void);
void delay_ms(u8 ms);

#define INT_SIZE 32  // Assumes 32-bit integer size

//------------------------------------------------------------------------------
//CRC5的 MSB在前计算,函数返回的CRC5 是传输值,需要将低5位倒序后得到实际值！
//输入:
//	dwInput		用于计算CRC5的源数据(右对齐格式)
//	iBitcnt		dwInput中需要计算的二进制位数,即dwInput.bit[iBitcnt-1]:bit0;
//输出:
//	CRC5的传输值
//------------------------------------------------------------------------------
uint32_t CRC5_MSBfirst(uint32_t dwInput, int iBitcnt) 
{
const	uint32_t poly5	= (0x05uL << (INT_SIZE - 5));			//CRC5 多项式扩展为32位,占用最高5位
		uint32_t crc5	= (0x1fuL << (INT_SIZE - 5));			//初始值扩展为32位,占用最高5位
		uint32_t udata	= (dwInput << (INT_SIZE - iBitcnt));	//输入数据扩展为32位, 低 iBitcnt 补0

	if(	(iBitcnt < 1) || (iBitcnt > INT_SIZE) )
	{	//iBitcnt 参数出错
		return 0xffffffff;
	}

	while(iBitcnt--)
	{
		if((udata ^ crc5) & (0x1uL << (INT_SIZE - 1)))
		{	// bit4 != bit4?
			crc5 <<= 1;
			crc5 ^= poly5;
		}
		else
		{
			crc5 <<= 1;
		}

		udata <<= 1;
	}

	//移动到原来的右对齐格式
	crc5 >>= (INT_SIZE - 5);

	//数据位翻转,产生CRC5
	crc5 ^= 0x1f;

	return crc5;
}// CRC5()


//------------------------------------------------------------------------------
//CRC5 的小端模式的版本,函数返回的是 CRC5 的实际值!
//输入:
//	dwInput		用于计算CRC5的源数据(右对齐格式)
//	iBitcnt		dwInput中需要计算的二进制位数,即dwInput.bit[iBitcnt-1]:bit0;
//输出:
//	CRC5的实际值(就是USB传输的CRC5的倒序格式)
//------------------------------------------------------------------------------
uint32_t CRC5_LSBfirst(uint32_t dwInput, int iBitcnt)
{
const	uint32_t poly5	= 0x14;
		uint32_t crc5	= 0x1f;
		uint32_t udata	= dwInput;

	if( (iBitcnt < 1) || (iBitcnt > INT_SIZE) )
	{	//iBitcnt 参数出错
		return 0xffffffff;
	}

	while(iBitcnt--)
	{
		if((udata ^ crc5) & 0x01)
		{
			crc5 >>= 1;
			crc5 ^= poly5;
		}
		else
		{
			crc5 >>= 1;
		}

		udata >>= 1;
	}

	//数据位翻转,产生CRC5
	crc5 ^= 0x1f;

	return crc5;
}// CRC5()


/* This function is based on code posted by John Sullivan to Wireshark-dev
 * mailing list on Jul 21, 2019.
 *
 * "One of the properties of LFSRs is that a 1 bit in the input toggles a
 *  completely predictable set of register bits *at any point in the
 *  future*. This isn't often useful for most CRC caculations on variable
 *  sized input, as the cost of working out which those bits are vastly
 *  outweighs most other methods."
 *
 * In USB 2.0, the CRC5 is calculated on either 11 or 19 bits inputs,
 * and thus this approach is viable.
 */
//此函数被 crc5_usb_11bit_input() 调用!
static uint8_t crc5_usb_bits(uint32_t v, int vl, uint8_t ival)
{

	
	uint8_t rv = ival;
	uint8_t i;
    static const uint8_t bvals[19] =
    {
		0x1e, 0x15, 0x03, 0x06, 0x0c, 0x18, 0x19, 0x1b,
        0x1f, 0x17, 0x07, 0x0e, 0x1c, 0x11, 0x0b, 0x16,
        0x05, 0x0a, 0x14
    };
    
    for(i = 0; i < vl; i++) 
	{
		if(v & (1uL << i)) 
		{
			rv ^= bvals[19 - vl + i];
        }
    }
    return rv;
}


//----------------------------------------------------------
//USB CRC5 格式的计算,输入数据的低11位参与计算
//输入:
//	dwInput	用于计算CRC5的源数据(右对齐格式),bit10:0参与计算 
//输出:
//	USB CRC5 的实际值(就是USB传输的CRC5的倒序格式)
//----------------------------------------------------------
uint8_t crc5_usb_11bit_input(uint32_t input)
{
    return crc5_usb_bits(input, 11, 0x02);
}


/******************** 串口打印函数 ********************/
void UartInit(void)
{
	P_SW1 = (P_SW1 & 0x3f) | 0x80;    //USART1 switch to, 0x00: P3.0 P3.1, 0x40: P3.6 P3.7, 0x80: P1.6 P1.7, 0xC0: P4.3 P4.4
	SCON = (SCON & 0x3f) | 0x40; 
	AUXR |= T1x12;          //定时器时钟1T模式
	AUXR &= ~S1BRT;          //串口1选择定时器1为波特率发生器
	TL1 = TM;
	TH1 = TM>>8;
	TR1 = 1;			//定时器1开始计时
}

void UartPutc(unsigned char dat)
{
	SBUF = dat; 
	while(TI==0);
	TI = 0;
}

char putchar(char c)
{
	UartPutc(c);
	return c;
}

/**********************************************/
void main(void)
{
//测试1)
//uint8_t addr = 0x20;	//7位USB设备地址值,实际的ADDR是2; 【7位地址 2=000 0010, 传输中LSB在前,=010 0000=0x20，计算CRC5时都使用倒序后的传输值】
//uint8_t endp = 0;		//4位端点地址
//
//测试2)
//uint8_t addr = 0x74;	//7位USB设备地址值,实际的ADDR是23;【7位地址 23=0x17=001 0111, 传输中LSB在前,=111 0100=0x74，计算CRC5时都使用倒序后的传输值】
//uint8_t endp = 0;		//4位端点地址
//
//测试3)
//uint8_t addr = 0x00;	//7位USB设备地址值,实际的ADDR是0;【7位地址 23=0x17=000 0000, 传输中LSB在前,=000 0000=0x00，计算CRC5时都使用倒序后的传输值】
//uint8_t endp = 0;		//4位端点地址
//
//测试4)
//uint8_t addr = 0x40;	//7位USB设备地址值,实际的ADDR是1;【7位地址 1=0x01=000 0001, 传输中LSB在前,=100 0000=0x40，计算CRC5时都使用倒序后的传输值】
//uint8_t endp = 0x08;	//4位端点地址, 实际值是1; 【4位端点地址 1=0x01=0001, 传输中LSB在前,=1000=0x08，计算CRC5时都使用倒序后的传输值】
//......................
uint8_t addr = 0x0f;	//7位USB设备地址值,实际的ADDR是1;【7位地址 1=0x01=000 0001, 传输中LSB在前,=100 0000=0x40，计算CRC5时都使用倒序后的传输值】
uint8_t endp = 0x02;	//4位端点地址, 实际值是1; 【4位端点地址 1=0x01=0001, 传输中LSB在前,=1000=0x08，计算CRC5时都使用倒序后的传输值】

	
uint32_t combined_data = ((uint32_t)addr << 4) | (endp & 0x0F);	//组合为11位计算CRC5的源数据(占用低11位) 
uint8_t	result;
uint8_t x;


    WTST = 0;  //设置程序指令延时参数，赋值为0可将CPU执行指令的速度设置为最快
    P_SW2 |= EAXFR; //扩展寄存器(XFR)访问使能
    CKCON = 0; //提高访问XRAM速度

    RSTFLAG |= 0x04;   //设置硬件复位后需要检测P3.2的状态选择运行区域，否则硬件复位后进入USB下载模式

	P0M1 = 0x00;   P0M0 = 0x00;   //设置为准双向口
	P1M1 = 0x00;   P1M0 = 0x00;   //设置为准双向口
	P2M1 = 0x00;   P2M0 = 0x00;   //设置为准双向口
	P3M1 = 0x00;   P3M0 = 0x00;   //设置为准双向口
	P4M1 = 0x00;   P4M0 = 0x00;   //设置为准双向口
	P5M1 = 0x00;   P5M0 = 0x00;   //设置为准双向口
	P6M1 = 0x00;   P6M0 = 0x00;   //设置为准双向口
	P7M1 = 0x00;   P7M0 = 0x00;   //设置为准双向口
	
    usb_init();
    Timer0_Init();

	UartInit();
    IE2 |= EUSB;   //IE2相关的中断位操作使能后，需要重新设置EUSB
	EA = 1;

	while (1)
	{
		delay_ms(250);					//每秒计算并上送一次
		delay_ms(250);
		delay_ms(250);
		delay_ms(250);


		printf("combined_data: 0x%04X\n", combined_data);	//组合源数据

    	result = CRC5_MSBfirst(combined_data, 11);			//大端模式(MSB在前,左对齐)的CRC5计算
		printf("MSB CRC5: 0x%02X\n", result);				//显示当前CRC5

		result = CRC5_LSBfirst(combined_data, 11);			//小端模式(LSB在后,右对齐)的CRC5计算
		printf("LSB CRC5: 0x%02X\n", result);				//显示当前CRC5

		x = crc5_usb_11bit_input(combined_data);			//USB中11位数据输入得CRC5计算,
		printf("crc5_usb_11bit_input LSB: 0x%02X\n", x);	//显示当前CRC5		

		printf("\r\n");
	}
}


//========================================================================
// 函数: void delay_ms(u8 ms)
// 描述: 延时函数。
// 参数: ms,要延时的ms数, 这里只支持1~255ms. 自动适应主时钟.
//========================================================================
void delay_ms(u8 ms)
{
	u16 i;
	do
	{
		i = MAIN_Fosc / 6000;
		while(--i);
	}while(--ms);
}

//========================================================================
// 函数: void Timer0_Init(void)
// 描述: 定时器0初始化函数。
//========================================================================
void Timer0_Init(void)
{
    // Timer0初始化
    TMOD &= 0xf0;//16 bits timer auto-reload
    AUXR |= T0x12;  //Timer0 set as 1T
    TH0 = (u8)(Timer0_Reload / 256);
    TL0 = (u8)(Timer0_Reload % 256);
    ET0 = 1;    //Timer0 interrupt enable
    TR0 = 1;    //Tiner0 run
}

//========================================================================
// 函数: void KeyResetScan(void)
// 描述: P3.2口按键长按1秒触发软件复位，进入USB下载模式。
//========================================================================
void KeyResetScan(void)
{
    if(!P32)
    {
        if(!Key_Flag)
        {
            Key_cnt++;
            if(Key_cnt >= 1000)		//连续1000ms有效按键检测
            {
                Key_Flag = 1;		//设置按键状态，防止重复触发

                USBCON = 0x00;      //清除USB设置
                USBCLK = 0x00;
                IRC48MCR = 0x00;
                
                delay_ms(10);
                IAP_CONTR = 0x60;   //触发软件复位，从ISP开始执行
                while (1);
            }
        }
    }
    else
    {
        Key_cnt = 0;
        Key_Flag = 0;
    }
}

//========================================================================
// 函数: void timer0_int(void)
// 描述: Timer0 1ms中断函数。
//========================================================================
void timer0_int(void) interrupt 1
{
    if (bUsbOutReady) //USB调试及复位所需代码
    {
        //USB_SendData(UsbOutBuffer,64);    //发送数据缓冲区，长度
        
        usb_OUT_done();
    }

    KeyResetScan();   //P3.2口按键触发软件复位，进入USB下载模式，不需要此功能可删除本行代码
}

//End Of File
